// ignore_for_file: no_leading_underscores_for_local_identifiers
part of 'core.dart';

/// @nodoc
class ZegoUIKitCoreMessage {
  void onIMRecvBroadcastMessage(
    String roomID,
    List<ZegoBroadcastMessageInfo> _messageList,
  ) {
    for (final _message in _messageList) {
      final message = ZegoInRoomMessage.fromBroadcastMessage(_message);
      ZegoUIKitCore.shared.coreData.streamControllerRemoteMessage?.add(message);
      ZegoUIKitCore.shared.coreData.messageList.add(message);
    }

    if (ZegoUIKitCore.shared.coreData.messageList.length > 500) {
      ZegoUIKitCore.shared.coreData.messageList.removeRange(
          0, ZegoUIKitCore.shared.coreData.messageList.length - 500);
    }

    ZegoUIKitCore.shared.coreData.streamControllerMessageList?.add(
        List<ZegoInRoomMessage>.from(
            ZegoUIKitCore.shared.coreData.messageList));
  }

  void clear() {
    ZegoUIKitCore.shared.coreData.messageList.clear();
    ZegoUIKitCore.shared.coreData.streamControllerMessageList?.add(
      List<ZegoInRoomMessage>.from(ZegoUIKitCore.shared.coreData.messageList),
    );
  }

  /// @return Error code, please refer to the error codes document https://docs.zegocloud.com/en/5548.html for details.
  Future<int> sendBroadcastMessage(String message) async {
    ZegoUIKitCore.shared.coreData.localMessageId =
        ZegoUIKitCore.shared.coreData.localMessageId - 1;

    final messageItem = ZegoInRoomMessage(
      messageID: ZegoUIKitCore.shared.coreData.localMessageId,
      user: ZegoUIKitCore.shared.coreData.localUser.toZegoUikitUser(),
      message: message,
      timestamp:
          ZegoUIKitCore.shared.coreData.networkDateTime_.millisecondsSinceEpoch,
    );
    messageItem.state.value = ZegoInRoomMessageState.idle;

    ZegoUIKitCore.shared.coreData.messageList.add(messageItem);
    ZegoUIKitCore.shared.coreData.streamControllerMessageList?.add(
      List<ZegoInRoomMessage>.from(ZegoUIKitCore.shared.coreData.messageList),
    );

    Future.delayed(const Duration(milliseconds: 300), () {
      if (ZegoInRoomMessageState.idle == messageItem.state.value) {
        /// if the status is still Idle after 300 ms,  it mean the message is not sent yet.
        messageItem.state.value = ZegoInRoomMessageState.sending;
        ZegoUIKitCore.shared.coreData.streamControllerMessageList?.add(
          List<ZegoInRoomMessage>.from(
              ZegoUIKitCore.shared.coreData.messageList),
        );
      }
    });

    return ZegoExpressEngine.instance
        .sendBroadcastMessage(ZegoUIKitCore.shared.coreData.room.id, message)
        .then((ZegoIMSendBroadcastMessageResult result) {
      messageItem.state.value = (result.errorCode == 0)
          ? ZegoInRoomMessageState.success
          : ZegoInRoomMessageState.failed;

      if (ZegoErrorCode.CommonSuccess == result.errorCode) {
        messageItem.messageID = result.messageID;
      }
      ZegoUIKitCore.shared.coreData.streamControllerLocalMessage
          ?.add(messageItem);

      ZegoUIKitCore.shared.coreData.streamControllerMessageList?.add(
        List<ZegoInRoomMessage>.from(ZegoUIKitCore.shared.coreData.messageList),
      );

      return result.errorCode;
    });
  }

  /// @return Error code, please refer to the error codes document https://docs.zegocloud.com/en/5548.html for details.
  Future<int> resendInRoomMessage(ZegoInRoomMessage message) async {
    ZegoUIKitCore.shared.coreData.messageList.removeWhere(
      (element) => element.messageID == message.messageID,
    );
    return sendBroadcastMessage(message.message);
  }
}
